/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.animation;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import ej.annotation.Nullable;
import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.bon.Util;
import ej.components.dependencyinjection.ServiceLoaderFactory;

/**
 * An animator executes animations at a fixed rate.
 */
public class Animator {

	/**
	 * The default tick period.
	 */
	public static final long DEFAULT_PERIOD = 50;
	/**
	 * The property used to define the period through the system properties.
	 * <p>
	 * Setting this property with a valid long value will override the default period of the animators.
	 */
	public static final String PERIOD_PROPERTY = "ej.animation.Animator.period"; //$NON-NLS-1$

	private static final String EMPTY_MESSAGE = ""; //$NON-NLS-1$

	private final List<Animation> animations;
	private boolean hasChanged;
	private Animation[] animationsCache;

	private long period;
	@Nullable
	private TimerTask task;

	/**
	 * Creates an animator.
	 * <p>
	 * Its period will be:
	 * <ol>
	 * <li>the value of the {@link #PERIOD_PROPERTY} system property if set and valid,</li>
	 * <li>the {@link #DEFAULT_PERIOD}.</li>
	 * </ol>
	 */
	public Animator() {
		this.animations = new ArrayList<>(1);
		this.hasChanged = true;
		this.period = initializePeriod();
		this.animationsCache = new Animation[0];
	}

	private static long initializePeriod() {
		String periodProperty = System.getProperty(PERIOD_PROPERTY);
		try {
			if (periodProperty != null) {
				return Long.parseLong(periodProperty);
			}
		} catch (NumberFormatException e) {
			Logger.getLogger(Animator.class.getName()).log(Level.WARNING, EMPTY_MESSAGE, e);
		}
		return DEFAULT_PERIOD;
	}

	/**
	 * Sets the period.
	 *
	 * @param period
	 *            the period to set.
	 */
	public void setPeriod(long period) {
		this.period = period;
		synchronized (this.animations) {
			if (this.task != null) {
				stop();
				start();
			}
		}
	}

	/**
	 * Gets the period.
	 *
	 * @return the period.
	 */
	public long getPeriod() {
		return this.period;
	}

	/**
	 * Starts an animation.
	 * <p>
	 * The animation tick method is then called every period.
	 *
	 * @param animation
	 *            the animation to start.
	 * @see Animation#tick(long)
	 * @see #getPeriod()
	 */
	public void startAnimation(Animation animation) {
		synchronized (this.animations) {
			this.animations.add(animation);
			this.hasChanged = true;
			start();
		}
	}

	/**
	 * Stops an animation.
	 * <p>
	 * The animation tick method will not be called anymore by this animator.
	 *
	 * @param animation
	 *            the animation to stop.
	 */
	public void stopAnimation(Animation animation) {
		synchronized (this.animations) {
			this.animations.remove(animation);
			this.hasChanged = true;
			if (this.animations.isEmpty()) {
				this.animationsCache = new Animation[0];
				stop();
			}
		}
	}

	/**
	 * Gets a cache of the set of animations to avoid concurrent modification.
	 *
	 * @return the cached animations set.
	 */
	private Animation[] getAnimations() {
		synchronized (this.animations) {
			if (this.hasChanged) {
				this.hasChanged = false;
				this.animationsCache = this.animations.toArray(new Animation[this.animations.size()]);
			}
		}
		return this.animationsCache;
	}

	private void tick() {
		long currentTimeMillis = Util.platformTimeMillis();
		Animation[] animations = getAnimations();
		for (Animation animation : animations) {
			try {
				if (!animation.tick(currentTimeMillis)) {
					stopAnimation(animation);
				}
			} catch (Throwable e) {
				Logger.getLogger(Animator.class.getName()).log(Level.WARNING, EMPTY_MESSAGE, e);
				stopAnimation(animation);
			}
		}
	}

	// MUST be called while owning the monitor on this.animations
	private void start() {
		if (this.task == null) {
			this.task = new TimerTask() {
				@Override
				public void run() {
					tick();
				}
			};
			ServiceLoaderFactory.getServiceLoader().getService(Timer.class, Timer.class).schedule(this.task,
					this.period, this.period);
		}
	}

	// MUST be called while owning the monitor on this.animations
	private void stop() {
		TimerTask task = this.task;
		if (task != null) {
			task.cancel();
			this.task = null;
		}
	}

}
