/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 *This library is provided in source code for use, modification and test, subject to license terms.
 *Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.automaton;

import java.util.logging.Level;
import java.util.logging.Logger;

import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.watchdog.EventsFeeder;
import ej.watchdog.Watchdog;
import ej.watchdog.WatchdogListener;
import ej.watchdog.internal.SimpleWatchdog;

/**
 * Starts an automaton that simulates user interaction.
 * <p>
 * It can be started immediately or after a period of inactivity.
 */
public class AutomatonManager {

	private Automaton automaton;
	private Watchdog watchdog;

	private TimerTask task;
	private final Timer timer;
	private final EventsFeeder feeder;

	/**
	 * Creates an automaton manager for an Automaton with a {@link SimpleWatchdog} feeds by all Events.
	 *
	 * @param automaton
	 *            the automaton.
	 * @param delay
	 *            the delay before triggering the Automaton.
	 * @throws NullPointerException
	 *             if the given automaton is <code>null</code>.
	 */
	public AutomatonManager(Automaton automaton, long delay) {
		this(automaton, delay, ServiceLoaderFactory.getServiceLoader().getService(Timer.class, Timer.class));
	}

	/**
	 * Creates an automaton manager for an Automaton with a {@link SimpleWatchdog} feeds by all Events.
	 *
	 * @param automaton
	 *            the automaton.
	 * @param delay
	 *            the delay before triggering the Automaton.
	 *
	 * @param timer
	 *            the timer.
	 * @throws NullPointerException
	 *             if the given automaton or timer is <code>null</code>.
	 */
	public AutomatonManager(Automaton automaton, long delay, Timer timer) {
		if (timer == null) {
			throw new NullPointerException();
		}
		setAutomaton(automaton);
		setWatchdog(new SimpleWatchdog(delay, timer));
		this.feeder = new EventsFeeder(this.watchdog);
		this.timer = timer;
	}

	/**
	 * Sets a watchdog.
	 * <p>
	 * If the given watchdog is set, the automaton is started as soon as {@link #arm()} method is called.
	 *
	 * @param watchdog
	 *            the watchdog.
	 */
	public void setWatchdog(Watchdog watchdog) {
		if (this.watchdog != null) {
			// stop and unregister
			this.watchdog.stop();
			this.watchdog.setWatchdogListener(null);
		}
		this.watchdog = watchdog;
		if (watchdog != null) {
			watchdog.setWatchdogListener(new WatchdogListener() {// Test null.
				@Override
				public void onFeed() {
					stopAutomaton();
				}

				@Override
				public void onTrigger() {
					startAutomaton();
				}
			});
		}
	}

	/**
	 * Returns the watchdog of this automaton manager.
	 *
	 * @return the watchdog or <code>null</code> if not defined.
	 */
	public Watchdog getWatchdog() {
		return this.watchdog;
	}

	/**
	 * Sets the automaton.
	 *
	 * @param automaton
	 *            the automaton.
	 * @throws NullPointerException
	 *             if the given automaton is <code>null</code>.
	 */
	public void setAutomaton(Automaton automaton) {
		if (automaton == null) {
			throw new NullPointerException();
		}
		this.automaton = automaton;
	}

	/**
	 * Starts the automaton.
	 * <p>
	 * If a watchdog is declared, the automaton is started after a period of inactivity, and stopped when an activity is
	 * detected.
	 *
	 * @see Watchdog
	 */
	public void arm() {
		this.feeder.start();
		if (this.watchdog != null) {
			this.watchdog.arm();
		} else {
			startAutomaton();
		}
	}

	/**
	 * Stops the automaton.
	 */
	public void stop() {
		this.feeder.stop();
		stopAutomaton();
	}

	private synchronized void startAutomaton() {
		if (this.automaton != null && this.task == null) {
			try {
				this.automaton.onStart();
				if (this.automaton != null) {
					this.task = new TimerTask() {
						@Override
						public void run() {
							try {
								AutomatonManager.this.automaton.run();
							} catch (final Throwable t) {
								AutomatonManager.this.uncaughtException(t);
							}
						}
					};
					this.timer.schedule(this.task, 0, this.automaton.getPeriod());
				}
			} catch (final Throwable t) {
				uncaughtException(t);
			}
		}
	}

	private synchronized void stopAutomaton() {
		final TimerTask task = this.task;
		this.task = null;
		if (task != null) {
			try {
				this.automaton.onStop();
			} catch (final Throwable t) {
				uncaughtException(t);
			}
			task.cancel();
		}
	}

	private void uncaughtException(final Throwable t) {
		Logger.getLogger(AutomatonManager.class.getName()).log(Level.WARNING, t.getMessage(), t);
	}

}
