/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave;

public interface ZwaveProtocol {
	static final long SOFT_RESET_WAIT_PERIOD                           = 1500;
	static final long ACK_WAIT_PERIOD                                  = 1600;
	static final long RESPONSE_WAIT_PERIOD                             = 5000;
	static final long MAX_RETRANSMISSION                               = 2;

	/* Predefined Node ID's */
	/** broadcast */
	static final byte NODE_BROADCAST                                   = (byte) 0xFF;

	/* SerialAPI */
	static final byte SOF                                              = 0x01;
	static final byte ACK                                              = 0x06;
	static final byte NAK                                              = 0x15;
	static final byte CAN                                              = 0x18;

	static final byte REQUEST                                          = 0x0;
	static final byte RESPONSE                                         = 0x1;

	/* Function Ids */
	static final byte FUNC_ID_SERIAL_API_GET_INIT_DATA                 = 0x02;
	static final byte FUNC_ID_SERIAL_API_APPL_NODE_INFORMATION         = 0x03;
	static final byte FUNC_ID_APPLICATION_COMMAND_HANDLER              = 0x04;
	static final byte FUNC_ID_ZW_GET_CONTROLLER_CAPABILITIES           = 0x05;
	static final byte FUNC_ID_SERIAL_API_GET_CAPABILITIES              = 0x07;
	static final byte FUNC_ID_SERIAL_API_SOFT_RESET                    = 0x08;
	static final byte FUNC_ID_ZW_SEND_DATA                             = 0x13;
	static final byte FUNC_ID_ZW_SEND_DATA_ABORT                       = 0x16;
	static final byte FUNC_ID_MEMORY_GET_ID                            = 0x20;
	static final byte FUNC_ID_ZW_GET_NODE_PROTOCOL_INFO                = 0x41;
	static final byte FUNC_ID_ZW_ADD_NODE_TO_NETWORK                   = 0x4A;
	static final byte FUNC_ID_ZW_REMOVE_NODE_FROM_NETWORK              = 0x4B;

	/* Mode parameters to AddNodeToNetwork */
	static final byte ADD_NODE_ANY                                     = 0x01;
	static final byte ADD_NODE_STOP                                    = 0x05;
	static final byte ADD_NODE_STOP_FAILED                             = 0x06;
	static final byte ADD_NODE_MODE_MASK                               = 0x0F;
	static final byte ADD_NODE_OPTION_NORMAL_POWER                     = (byte) 0x80;
	static final byte ADD_NODE_OPTION_NETWORK_WIDE                     = 0x40;
	static final byte ADD_NODE_PARAMETERS                              = ADD_NODE_ANY |	ADD_NODE_OPTION_NORMAL_POWER | ADD_NODE_OPTION_NETWORK_WIDE;

	/* Callback states from ZW_AddNodeToNetwork */
	static final byte ADD_NODE_STATUS_LEARN_READY                      = 0x01;
	static final byte ADD_NODE_STATUS_NODE_FOUND                       = 0x02;
	static final byte ADD_NODE_STATUS_ADDING_SLAVE                     = 0x03;
	static final byte ADD_NODE_STATUS_ADDING_CONTROLLER                = 0x04;
	static final byte ADD_NODE_STATUS_PROTOCOL_DONE                    = 0x05;
	static final byte ADD_NODE_STATUS_DONE                             = 0x06;
	static final byte ADD_NODE_STATUS_FAILED                           = 0x07;
	static final byte ADD_NODE_STATUS_NOT_PRIMARY                      = 0x23;

	/* Mode parameters to ZW_RemoveNodeFromNetwork */
	static final byte REMOVE_NODE_ANY                                 = ADD_NODE_ANY;
	static final byte REMOVE_NODE_STOP                                = ADD_NODE_STOP;

	/* Callback states from ZW_RemoveNodeFromNetwork */
	static final byte REMOVE_NODE_STATUS_LEARN_READY                  = ADD_NODE_STATUS_LEARN_READY;
	static final byte REMOVE_NODE_STATUS_NODE_FOUND                   = ADD_NODE_STATUS_NODE_FOUND;
	static final byte REMOVE_NODE_STATUS_REMOVING_SLAVE               = ADD_NODE_STATUS_ADDING_SLAVE;
	static final byte REMOVE_NODE_STATUS_REMOVING_CONTROLLER          = ADD_NODE_STATUS_ADDING_CONTROLLER;
	static final byte REMOVE_NODE_STATUS_DONE                         = ADD_NODE_STATUS_DONE;
	static final byte REMOVE_NODE_STATUS_FAILED                       = ADD_NODE_STATUS_FAILED;

	/* Transmit frame option flags */
	/** request acknowledge from destination node */
	static final byte TRANSMIT_OPTION_ACK                              = 0x01;
	/** request retransmission via repeater nodes */
	static final byte TRANSMIT_OPTION_AUTO_ROUTE                       = 0x04;
	/** Use explore frame if needed */
	static final byte TRANSMIT_OPTION_EXPLORE                          = 0x20;
	/** Send data options */
	static final byte TRANSMIT_OPTIONS                                 = TRANSMIT_OPTION_EXPLORE | TRANSMIT_OPTION_AUTO_ROUTE | TRANSMIT_OPTION_ACK;

	/* Transmit complete codes */
	/** transmit complete */
	static final byte TRANSMIT_COMPLETE_OK                             = 0x00;
	/** retransmission error */
	static final byte TRANSMIT_COMPLETE_NO_ACK                         = 0x01;
	/** transmit error */
	static final byte TRANSMIT_COMPLETE_FAIL                           = 0x02;

	/**
	 * A response route is locked by the application
	 */
	static final byte RECEIVE_STATUS_ROUTED_BUSY    = 0x01;
	/**
	 * Received at low output power level, this must
	 * have the same value as TRANSMIT_OPTION_LOW_POWER
	 */
	static final byte RECEIVE_STATUS_LOW_POWER     = 0x02;
	/**
	 * Mask for masking out the received frametype bits
	 */
	static final byte RECEIVE_STATUS_TYPE_MASK     = 0x0C;
	/**
	 * Received frame is singlecast frame (rxOptions == xxxx00xx)
	 */
	static final byte RECEIVE_STATUS_TYPE_SINGLE   = 0x00;
	/**
	 * Received frame is broadcast frame  (rxOptions == xxxx01xx)
	 */
	static final byte RECEIVE_STATUS_TYPE_BROAD    = 0x04;
	/**
	 * Received frame is multicast frame (rxOptions == xxxx10xx)
	 * Only TYPE_BROAD can be active at the same time as TYPE_EXPLORE
	 */
	static final byte RECEIVE_STATUS_TYPE_MULTI    = 0x08;
	/**
	 * Received frame is an explore frame (rxOptions == xxx1xxxx)
	 * Only TYPE_BROAD can be active at the same time as TYPE_EXPLORE
	 */
	static final byte RECEIVE_STATUS_TYPE_EXPLORE  = 0x10;
	/**
	 * Received frame is not send to me (useful only in promiscuous mode)
	 */
	static final byte RECEIVE_STATUS_FOREIGN_FRAME  = 0x40;

	/*============================================================================
	 **    Defines for ApplicationNodeInformation field deviceOptionMask
	 **--------------------------------------------------------------------------*/
	static final byte APPLICATION_NODEINFO_NOT_LISTENING               = 0x00;
	static final byte APPLICATION_NODEINFO_LISTENING                   = 0x01;
	static final byte APPLICATION_NODEINFO_OPTIONAL_FUNCTIONALITY      = 0x02;
	static final byte APPLICATION_FREQ_LISTENING_MODE_1000ms           = 0x10;
	static final byte APPLICATION_FREQ_LISTENING_MODE_250ms            = 0x20;

	/* Basic Device Class identifiers */
	/** Node is a portable controller */
	static final byte BASIC_TYPE_CONTROLLER                            = 0x01;
	/** Node is a slave with routing capabilities */
	static final byte BASIC_TYPE_ROUTING_SLAVE                         = 0x04;
	/** Node is a slave */
	static final byte BASIC_TYPE_SLAVE                                 = 0x03;
	/** Node is a static controller */
	static final byte BASIC_TYPE_STATIC_CONTROLLER                     = 0x02;

	/*Specific Device Class Not Used*/
	static final byte SPECIFIC_TYPE_NOT_USED                           = 0x00;

	/* Generic and Specific Device Class identifiers */
	/** Device class Generic Controller */
	static final byte GENERIC_TYPE_GENERIC_CONTROLLER                  = 0x01;
	/** Remote Controller */
	static final byte SPECIFIC_TYPE_PORTABLE_REMOTE_CONTROLLER         = 0x01;
	/** Portable Scene Controller */
	static final byte SPECIFIC_TYPE_PORTABLE_SCENE_CONTROLLER          = 0x02;

	/* Device class Sensor Binary */
	/** Binary Sensor */
	static final byte GENERIC_TYPE_SENSOR_BINARY                       = 0x20;
	/** Routing Binary Sensor */
	static final byte SPECIFIC_TYPE_ROUTING_SENSOR_BINARY              = 0x01;

	/* Device class Static Controller */
	/** Static Controller */
	static final byte GENERIC_TYPE_STATIC_CONTROLLER                   = 0x02;
	/** Central Controller Device Type */
	static final byte SPECIFIC_TYPE_PC_CONTROLLER                      = 0x01;
	/** Scene Controller */
	static final byte SPECIFIC_TYPE_SCENE_CONTROLLER                   = 0x02;

	/* Device class Switch Binary */
	/** Binary Switch */
	static final byte GENERIC_TYPE_SWITCH_BINARY                       = 0x10;
	/** On/Off Power Switch Device Type */
	static final byte SPECIFIC_TYPE_POWER_SWITCH_BINARY                = 0x01;
	/**Binary Scene Switch */
	static final byte SPECIFIC_TYPE_SCENE_SWITCH_BINARY                = 0x03;
	/** Siren Device Type */
	static final byte SPECIFIC_TYPE_SIREN                              = 0x05;
	
	/* Device class Thermostat */
	/** Thermostat */ 
	static final byte GENERIC_TYPE_THERMOSTAT                          = 0x08;

	/* command class */
	static final byte COMMAND_CLASS_ASSOCIATION                        = (byte) 0x85;
	static final byte COMMAND_CLASS_BASIC                              = 0x20;
	static final byte COMMAND_CLASS_BATTERY                            = (byte) 0x80;
	static final byte COMMAND_CLASS_CONFIGURATION                      = 0x70;
	static final byte COMMAND_CLASS_MANUFACTURER_SPECIFIC              = 0x72;
	static final byte COMMAND_CLASS_MARK                               = (byte) 0xEF;
	static final byte COMMAND_CLASS_METER                              = 0x32;
	static final byte COMMAND_CLASS_SENSOR_ALARM                       = (byte) 0x9C;
	static final byte COMMAND_CLASS_SENSOR_BINARY                      = 0x30;
	static final byte COMMAND_CLASS_SWITCH_BINARY                      = 0x25;
	static final byte COMMAND_CLASS_VERSION                            = (byte) 0x86;
	static final byte COMMAND_CLASS_WAKE_UP                            = (byte) 0x84;
	static final byte COMMAND_CLASS_THERMOSTAT_MODE                    = 0x40;
	static final byte COMMAND_CLASS_SENSOR_MULTILEVEL                  = 0x31;
	static final byte COMMAND_CLASS_THERMOSTAT_SETPOINT                = 0x43;

	/* Association command class commands */
	static final byte ASSOCIATION_GET                                  = 0x02;
	static final byte ASSOCIATION_GROUPINGS_GET                        = 0x05;
	static final byte ASSOCIATION_GROUPINGS_REPORT                     = 0x06;
	static final byte ASSOCIATION_REMOVE                               = 0x04;
	static final byte ASSOCIATION_REPORT                               = 0x03;
	static final byte ASSOCIATION_SET                                  = 0x01;

	/* Basic command class commands */
	static final byte BASIC_GET                                        = 0x02;
	static final byte BASIC_REPORT                                     = 0x03;
	static final byte BASIC_SET                                        = 0x01;

	/* Meter command class commands */
	static final byte METER_GET                                        = 0x01;
	static final byte METER_REPORT                                     = 0x02;

	static final byte SENSOR_BINARY_GET                                = 0x02;
	static final byte SENSOR_BINARY_REPORT                             = 0x03;
	/* Values used for Sensor Binary Report command */
	static final byte SENSOR_BINARY_REPORT_IDLE                        = 0x00;
	static final byte SENSOR_BINARY_REPORT_DETECTED_AN_EVENT           = (byte) 0xFF;

	/* Switch Binary command class commands */
	static final byte SWITCH_BINARY_GET                                = 0x02;
	static final byte SWITCH_BINARY_REPORT                             = 0x03;
	static final byte SWITCH_BINARY_SET                                = 0x01;
	
	/* Thermostat command class commands */
	static final byte THERMOSTAT_MODE_GET                              = 0x02;
	static final byte THERMOSTAT_MODE_REPORT                           = 0x03;
	static final byte THERMOSTAT_MODE_SET                              = 0x01;
	static final byte THERMOSTAT_MODE_SUPPORTED_GET                    = 0x04;
	static final byte THERMOSTAT_MODE_SUPPORTED_REPORT                 = 0x05;
	
	/* Thermostat mode */
	static final byte THERMOSTAT_MODE_OFF                    = 0x00;
	static final byte THERMOSTAT_MODE_HEAT                   = 0x01;
	static final byte THERMOSTAT_MODE_COOL                   = 0x02;
	static final byte THERMOSTAT_MODE_AUTO                   = 0x03;
	static final byte THERMOSTAT_MODE_AUXILIARY              = 0x04;
	static final byte THERMOSTAT_MODE_RESUME                 = 0x05;
	static final byte THERMOSTAT_MODE_FAN                    = 0x06;
	static final byte THERMOSTAT_MODE_FURNACE                = 0x07;
	static final byte THERMOSTAT_MODE_DRY                    = 0x08;
	static final byte THERMOSTAT_MODE_MOIST                  = 0x09;
	static final byte THERMOSTAT_MODE_AUTO_CHANGEOVER        = 0x0A;
	static final byte THERMOSTAT_MODE_ENERGY_HEAT            = 0x0B;
	static final byte THERMOSTAT_MODE_ENERGY_COOL            = 0x0C;
	static final byte THERMOSTAT_MODE_AWAY                   = 0x0D;
	static final byte THERMOSTAT_MODE_FULL_POWER             = 0x0F;

	static final byte THERMOSTAT_BITMASK_1_MODE_OFF                    = 0b00000001;
	static final byte THERMOSTAT_BITMASK_1_MODE_HEAT                   = 0b00000010;
	static final byte THERMOSTAT_BITMASK_1_MODE_COOL                   = 0b00000100;
	static final byte THERMOSTAT_BITMASK_1_MODE_AUTO                   = 0b00001000;
	static final byte THERMOSTAT_BITMASK_1_MODE_AUXILIARY              = 0b00010000;
	static final byte THERMOSTAT_BITMASK_1_MODE_RESUME                 = 0b00100000;
	static final byte THERMOSTAT_BITMASK_1_MODE_FAN                    = 0b01000000;
	static final byte THERMOSTAT_BITMASK_1_MODE_FURNACE                = (byte) 0b10000000;
	static final byte THERMOSTAT_BITMASK_2_MODE_DRY                    = 0b00000001;
	static final byte THERMOSTAT_BITMASK_2_MODE_MOIST                  = 0b00000010;
	static final byte THERMOSTAT_BITMASK_2_MODE_AUTO_CHANGEOVER        = 0b00000100;
	static final byte THERMOSTAT_BITMASK_2_MODE_ENERGY_HEAT            = 0b00001000;
	static final byte THERMOSTAT_BITMASK_2_MODE_ENERGY_COOL            = 0b00010000;
	static final byte THERMOSTAT_BITMASK_2_MODE_AWAY                   = 0b01000000;
	static final byte THERMOSTAT_BITMASK_3_MODE_FULL_POWER             = 0b00000001;
	
	/* Thermostat Mode SetPoint */
	static final byte THERMOSTAT_SETPOINT_SET                          = 0x01;
	static final byte THERMOSTAT_SETPOINT_GET                          = 0x02;
	static final byte THERMOSTAT_SETPOINT_REPORT                       = 0x03;
	
	/* Sensor Multilevel */
	static final byte SENSOR_MULTILEVEL_GET                            = 0x04;
	static final byte SENSOR_MULTILEVEL_REPORT                         = 0x05;
	
	/* Wake Up command class commands */
	static final byte WAKE_UP_INTERVAL_GET                             = 0x05;
	static final byte WAKE_UP_INTERVAL_REPORT                          = 0x06;
	static final byte WAKE_UP_INTERVAL_SET                             = 0x04;
	static final byte WAKE_UP_NO_MORE_INFORMATION                      = 0x08;
	static final byte WAKE_UP_NOTIFICATION                             = 0x07;
}
