/*
 * Copyright 2013-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.components.registry.impl;

import ej.annotation.Nullable;
import ej.basictool.map.PackedMap;
import ej.components.registry.BundleActivator;
import ej.components.registry.BundleRegistry;

/**
 * Default implementation of the bundle registry.
 */
public abstract class AbstractRegistry implements BundleActivator, BundleRegistry {

	private State currentState;
	private final PackedMap<Class<?>, Object> services;
	private BundleActivator[] loadedBundles;

	/**
	 * Creates a new registry.
	 */
	public AbstractRegistry() {
		this.currentState = State.Created;
		this.services = new PackedMap<>();
		this.loadedBundles = new BundleActivator[0];
	}

	/**
	 * Gets the loaded bundles.
	 *
	 * @return the loaded bundles.
	 */
	protected abstract BundleActivator[] loadBundles();

	@Override
	public State getCurrentState() {
		return this.currentState;
	}

	@Override
	public void initialize() {
		checkState(State.Created);
		// load bundles
		BundleActivator[] bundles = loadBundles();
		int bundlesLength = bundles.length;
		register(BundleRegistry.class, this);
		this.loadedBundles = bundles;
		for (int c = -1; ++c < bundlesLength;) {
			BundleActivator bundle = bundles[c];
			try {
				bundle.initialize();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		this.currentState = State.Initialized;
	}

	@Override
	public void link() {
		checkState(State.Initialized);
		BundleActivator[] bundles = this.loadedBundles;
		int bundlesLength = bundles.length;
		for (int c = -1; ++c < bundlesLength;) {
			BundleActivator bundle = bundles[c];
			try {
				bundle.link();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		this.currentState = State.Linked;
	}

	@Override
	public void start() {
		checkState(State.Linked);
		BundleActivator[] bundles = this.loadedBundles;
		int bundlesLength = bundles.length;
		for (int c = -1; ++c < bundlesLength;) {
			BundleActivator bundle = bundles[c];
			try {
				bundle.start();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		this.currentState = State.Started;
	}

	@Override
	public void stop() {
		checkState(State.Started);
		BundleActivator[] bundles = this.loadedBundles;
		int bundlesLength = bundles.length;
		for (int c = -1; ++c < bundlesLength;) {
			BundleActivator bundle = bundles[c];
			try {
				bundle.stop();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		this.currentState = State.Stopped;
		this.loadedBundles = new BundleActivator[0];
		this.services.clear();
	}

	@Override
	public void reset() {
		if (this.currentState == State.Started) {
			stop();
		}
		this.currentState = State.Created;
	}

	@Override
	public <T> void register(Class<T> service, T implementation) {
		checkState(State.Created);
		// TODO check service already registered?
		// TODO check null?
		this.services.put(service, implementation);
	}

	@Override
	@Nullable
	public <T> T getService(Class<T> service) {
		if (this.currentState == State.Created || this.currentState == State.Stopped) {
			throw new IllegalStateException();
		}
		@SuppressWarnings("unchecked")
		PackedMap<Class<?>, Object> services = this.services;
		T implementation = (T) services.get(service); // ensured by construction: see register(Class<T>, T)
		return implementation;
	}

	private void checkState(State state) {
		if (this.currentState != state) {
			throw new IllegalStateException();
		}
	}

}
