/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.data;

/**
 * This class defines methods to maintain the current state of the serialized/deserialized data.
 */
class DataStreamState{

	/**
	 * Default maximum number of nested collections.
	 * Can be overridden at runtime by setting the system property <code>ej.data.maxCollectionDepth</code>.
	 */
	private static final int DEFAULT_MAX_COLLECTION_DEPTH = 10;

	/**
	 * @see DataStreamState
	 */
	private final byte[] collectionsStateStack;
	private int collectionsStateStackPtr;


	public DataStreamState(){
		this.collectionsStateStack = new byte[Integer.getInteger(new StringBuilder().append("ej").append('.').append("data").append('.').append("maxCollectionDepth").toString(), DEFAULT_MAX_COLLECTION_DEPTH)];
		this.collectionsStateStack[this.collectionsStateStackPtr = 0] = initialize(VALUE_ROOT);
	}

	public boolean isRoot() {
		return collectionsStateStackPtr == 0;
	}

	public boolean isEOF() {
		return collectionsStateStackPtr == -1;
	}

	public boolean isWithinCollection() {
		return collectionsStateStackPtr > 0;
	}

	/**
	 * @param nextCollectionState
	 */
	public void push(byte newState) {
		collectionsStateStack[++collectionsStateStackPtr] = newState;
	}

	public void setCurrentState(byte newState) {
		collectionsStateStack[collectionsStateStackPtr] = newState;
	}

	public void pop() {
		--collectionsStateStackPtr;
	}

	/**
	 * @return the current state
	 * @throws IllegalStateException if no current state
	 */
	public byte getCurrentState() {
		int topIndex = collectionsStateStackPtr;
		if(topIndex >= 0){
			return collectionsStateStack[topIndex];
		}
		else{
			throw new IllegalStateException();
		}
	}

	static final int KEY_INT 			= 0;
	static final int KEY_STRING 		= 1;
	static final int VALUE_BT		 	= 2;
	static final int VALUE_ARRAY	 	= 3;
	static final int VALUE_MAP		 	= 4;
	static final int VALUE_ROOT		 	= 5;


	static final int KEY_TYPE_NOT_SET 	= 0;
	static final int KEY_TYPE_INT 		= 1;
	static final int KEY_TYPE_STRING 	= 2;

	static final int NEXT_IS_KEY_FALSE 	= 0;
	static final int NEXT_IS_KEY_TRUE 	= 1;

	private static final int TYPE_SIZE			= 3;
	private static final int TYPE_OFFSET		= 0x0;
	private static final int TYPE_MASK			= (1<<TYPE_SIZE)-1;
	private static final int KEY_TYPE_SIZE		= 2;
	private static final int KEY_TYPE_MASK		= (1<<KEY_TYPE_SIZE)-1;
	private static final int KEY_TYPE_OFFSET	= TYPE_OFFSET+TYPE_SIZE;
	private static final int NEXT_IS_KEY_SIZE	= 1;
	private static final int NEXT_IS_KEY_MASK	= (1<<NEXT_IS_KEY_SIZE)-1;
	private static final int NEXT_IS_KEY_OFFSET	= KEY_TYPE_OFFSET+KEY_TYPE_SIZE;

	public static byte initialize(int type) {
		byte result = 0;
		result = setNextIsKey(result, false);
		result = setType(result, type);
		result = setKeyType(result, KEY_TYPE_NOT_SET);
		return result;
	}

	public static byte setNextIsKey(byte collectionState, boolean value) {
		return (byte)((collectionState&~(NEXT_IS_KEY_MASK<<NEXT_IS_KEY_OFFSET)) | (value ? NEXT_IS_KEY_TRUE : NEXT_IS_KEY_FALSE)<<NEXT_IS_KEY_OFFSET);
	}

	public static byte setType(byte collectionState, int type) {
		return (byte)((collectionState&~(TYPE_MASK<<TYPE_OFFSET)) | type<<TYPE_OFFSET);
	}

	public static byte setKeyType(byte collectionState, int keyType) {
		return (byte)((collectionState&~(KEY_TYPE_MASK<<KEY_TYPE_OFFSET)) | keyType<<KEY_TYPE_OFFSET);
	}

	public static int getType(byte collectionState) {
		return (collectionState>>>TYPE_OFFSET) & TYPE_MASK;
	}

	public static int getKeyType(byte collectionState) {
		return (collectionState>>>KEY_TYPE_OFFSET) & KEY_TYPE_MASK;
	}

	public static boolean getNextIsKey(byte collectionState) {
		return ((collectionState>>>NEXT_IS_KEY_OFFSET) & NEXT_IS_KEY_MASK) == NEXT_IS_KEY_TRUE ? true : false;
	}


	public void checkAndUpdateState(int type){
		byte collectionState = getCurrentState();
		// within a collection (array or map or root) check type allowed as a map key
		int collectionType = getType(collectionState);
		if(collectionType == VALUE_MAP){
			boolean nextIsKey = getNextIsKey(collectionState);
			if(nextIsKey){
				boolean allowedKey = (type == KEY_INT || type == KEY_STRING);
				if(!allowedKey){
					throw new IllegalStateException();
				}
				if(getKeyType(collectionState) == KEY_TYPE_NOT_SET){
					collectionState = setKeyType(collectionState, type);
					setCurrentState(collectionState);
				}
				else{
					if(getKeyType(collectionState) != type){
						throw new IllegalStateException();
					}
				}
			}
			else{
				boolean allowedKey = (type != KEY_INT && type != KEY_STRING);
				if(!allowedKey){
					throw new IllegalStateException();
				}
			}
			collectionState = setNextIsKey(collectionState, !nextIsKey);
			setCurrentState(collectionState);
		}
		else if(collectionType == VALUE_ARRAY || collectionType == VALUE_ROOT){
			if(type == KEY_INT || type == KEY_STRING){
				throw new IllegalStateException();
			}
		}
		else{
			throw new AssertionError();
		}

		if(type == VALUE_MAP || type == VALUE_ARRAY){
			byte nextCollectionState = initialize(type);
			if(type == VALUE_MAP){
				nextCollectionState = setNextIsKey(nextCollectionState, true);
			}
			push(nextCollectionState);
		}
	}

	/**
	 * @param type either {@link DataStreamState#KEY_INT} or {@link DataStreamState#KEY_STRING}.
	 * @throws IllegalStateException on check inconsistency.
	 */
	public void checkAndUpdateStateEndCollection(int type){
		if(isWithinCollection()){
			byte collectionState = getCurrentState();
			if(getType(collectionState) != type) {
				throw new IllegalStateException();
			}
			if(type == VALUE_MAP){
				if(!getNextIsKey(collectionState)){
					throw new IllegalStateException();
				}
			}
			pop();
		}
		else{
			throw new IllegalStateException();
		}
	}


	/**
	 * @return true when end of data is reached, false otherwise.
	 */
	public boolean checkAndUpdateStateEndOfData(){
		if(isRoot()){
			pop();
			return true;
		}
		else{
			return false;
		}
	}

	public void checkSkipValue() {
		// skipValue() allowed everywhere in root/map/array except in map when a key is expected
		byte collectionState = getCurrentState();
		int collectionType = DataStreamState.getType(collectionState);
		if(collectionType == DataStreamState.VALUE_MAP){
			boolean nextIsKey = DataStreamState.getNextIsKey(collectionState);
			if(nextIsKey){
				throw new IllegalStateException();
			}
			collectionState = setNextIsKey(collectionState, !nextIsKey);
			setCurrentState(collectionState);
		}
	}
}
