/*
 * Java
 *
 * Copyright 2012-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.fp.event;

import ej.fp.EventQueue;
import ej.fp.FrontPanel;

/**
 * This class provides some methods to create internal MicroUI events which target the MicroUI
 * <code>EventGenerator</code> event generator implementation.
 * <p>
 * Several methods takes in argument a string <code>generatorID</code>. This ID must refer to a MicroUI events generator
 * defined during the MicroEJ plaform build in <code>microui/microui.xml</code> file (in platform configuration
 * project).
 * <p>
 * In designer mode (plugin mode), this ID is unknown so all methods throw an {@link IllegalArgumentException}. The
 * caller should catch this exception and prints in the standard output stream a description of this event.
 * <p>
 * In MicroEJ application mode, this exception should not occur (the MicroUI events generator ID should exist). However,
 * when exception occurs, the trace allows to the user to <i>note</i> there is an issue according the MicroUI events
 * generator ID.
 */
@SuppressWarnings("nls")
public class MicroUIEventGenerator {

	// MicroUI events representation
	private static final int EVENT_ID_MASK = 0xff;
	private static final int EVENT_ID_SHIFT = 24;
	private static final int EVENT_DATA_MASK = 0xffffff;
	private static final int EVENT_DATA_SHIFT = 0;

	private MicroUIEventGenerator() {
		// hide implicit constructor
	}

	/**
	 * Returns the type (i.e. the event generator id) of the event.
	 *
	 * @param event
	 *            the event to decode.
	 * @return the event's type.
	 */
	public static int getType(int event) {
		return (event >> EVENT_ID_SHIFT) & EVENT_ID_MASK;
	}

	/**
	 * Returns the data component of the event.
	 *
	 * @param event
	 *            the event to decode.
	 * @return the data component of the event.
	 */
	public static int getData(int event) {
		return (event >> EVENT_DATA_SHIFT) & EVENT_DATA_MASK;
	}

	/**
	 * Constructs and returns an event.
	 *
	 * @param generatorId
	 *            the required generator id (i.e. type) of the event.
	 * @param data
	 *            the data component of the event.
	 * @return the constructed event.
	 * @throws IllegalArgumentException
	 *             whether the generatorID refers to an unknown MicroUI generator (see class comment).
	 */
	public static int buildEvent(String generatorId, int data) throws IllegalArgumentException {
		return buildEvent(getEventGeneratorID(generatorId), data);
	}

	/**
	 * Constructs and returns an event.
	 *
	 * @param generatorId
	 *            the required generator id (i.e. type) of the event.
	 * @param data
	 *            the data component of the event.
	 * @return the constructed event.
	 */
	public static int buildEvent(int generatorId, int data) {
		return ((generatorId & EVENT_ID_MASK) << EVENT_ID_SHIFT) | ((data & EVENT_DATA_MASK) << EVENT_DATA_SHIFT);
	}

	/**
	 * Adds an event in the queue. This event must have been build using {@link #buildEvent(int, int)} or
	 * {@link #buildEvent(String, int)}.
	 *
	 * @param event
	 *            the event to add.
	 */
	public static void addEvent(int event) {
		getEventQueue().enqueudEvent(event);
	}

	/**
	 * Adds several events in the queue. The first event in the array must have been build using
	 * {@link #buildEvent(int, int)} or {@link #buildEvent(String, int)}.
	 *
	 * @param events
	 *            the events to add.
	 */
	public static void addEvents(int[] events) {
		getEventQueue().enqueudEvents(events);
	}

	/**
	 * Constructs an event and adds it to the simulator input queue. Exactly equivalent to
	 * {@code addEvent(buildEvent(generatorId, data))}.
	 *
	 * @param generatorId
	 *            the required generator id (i.e. type) of the event.
	 * @param data
	 *            the data component of the event.
	 * @throws IllegalArgumentException
	 *             whether the generatorID refers to an unknown MicroUI generator (see class comment).
	 */
	public static void addEvent(String generatorId, int data) throws IllegalArgumentException {
		addEvent(buildEvent(generatorId, data));
	}

	/**
	 * Constructs an event and adds it to the simulator input queue. Exactly equivalent to
	 * {@code addEvent(buildEvent(generatorId, data))}.
	 *
	 * @param generatorId
	 *            the required generator id (i.e. type) of the event.
	 * @param data
	 *            the data component of the event.
	 */
	public static void addEvent(int generatorId, int data) {
		addEvent(buildEvent(generatorId, data));
	}

	private static int getEventGeneratorID(String eventGenerator) throws IllegalArgumentException {
		try {
			String eventGeneratorID = System
					.getProperty("com.is2t.microbsp.microui.io.eventgenerator." + eventGenerator);
			return Integer.parseInt(eventGeneratorID);
		} catch (Exception e) {
			// missing or invalid ID: not specified ID in designer mode, or error in MicroEJ application mode
			throw new IllegalArgumentException("MicroUI events generator ID '" + eventGenerator + "' is unknown.");
		}
	}

	private static EventQueue getEventQueue() {
		return FrontPanel.getFrontPanel().getDevice().getEventQueue();
	}
}
