/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style;

import ej.annotation.NonNull;
import ej.annotation.Nullable;

/**
 * Represents an element.
 */
public interface Element {

	/**
	 * Empty element array. Avoid creating new instances.
	 *
	 * @see #getChildrenElements
	 */
	@NonNull
	Element[] EMPTY_ELEMENT_ARRAY = new Element[0];

	/**
	 * Gets whether or not the element has the given class selector.
	 *
	 * @param classSelector
	 *            the class selector to check.
	 * @return <code>true</code> if the element has the given class selector, <code>false</code> otherwise.
	 */
	boolean hasClassSelector(@NonNull String classSelector);

	/**
	 * Gets whether or not the element is in the given state.
	 *
	 * @param state
	 *            the state to check.
	 * @return <code>true</code> if the element is in the given state, <code>false</code> otherwise.
	 */
	boolean isInState(@NonNull State state);

	/**
	 * Gets the value of an attribute.
	 *
	 * @param attribute
	 *            the attribute to search.
	 * @return the value of the attribute or <code>null</code> if the attribute does not exist.
	 */
	@Nullable
	String getAttribute(@NonNull String attribute);

	/**
	 * Gets the parent element.
	 *
	 * @return the parent element
	 */
	@Nullable
	Element getParentElement();

	/**
	 * Gets the children elements.
	 * <p>
	 * Returns an empty array if no children.
	 *
	 * @return the children elements.
	 * @see #EMPTY_ELEMENT_ARRAY
	 */
	@NonNull
	Element[] getChildrenElements();

	/**
	 * Gets the child element at an index.
	 *
	 * @param index
	 *            the index of the element to get.
	 * @return the child element at the given index.
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range ((<code>index &lt; 0 || index &gt;= getChildrenCount()</code>)
	 */
	@NonNull
	Element getChild(int index);

	/**
	 * Gets the number of children of this element.
	 *
	 * @return the number of children.
	 */
	int getChildrenCount();

}
