/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.dimension;

import ej.annotation.Nullable;
import ej.mwt.MWT;
import ej.style.container.Rectangle;

/**
 * Dimension with constraint relative to the parent.
 */
public class RelativeDimension implements Dimension {

	private float widthPercent;
	private float heightPercent;

	/**
	 * Creates a relative dimension without constraint.
	 */
	public RelativeDimension() {
		this.widthPercent = -1;
		this.heightPercent = -1;
	}

	/**
	 * Creates a relative dimension with constraints.
	 *
	 * @param widthPercent
	 *            the width proportion to set.
	 * @param heightPercent
	 *            the height proportion to set.
	 */
	public RelativeDimension(float widthPercent, float heightPercent) {
		this.widthPercent = widthPercent;
		this.heightPercent = heightPercent;
	}

	/**
	 * Sets the width proportion.
	 *
	 * @param widthPercent
	 *            the width proportion to set.
	 */
	public void setWidthProportion(float widthPercent) {
		this.widthPercent = widthPercent;
	}

	/**
	 * Sets the height proportion.
	 *
	 * @param heightPercent
	 *            the height proportion to set.
	 */
	public void setHeightProportion(float heightPercent) {
		this.heightPercent = heightPercent;
	}

	/**
	 * Sets the width and height proportions.
	 *
	 * @param widthPercent
	 *            the width proportion to set.
	 * @param heightPercent
	 *            the height proportion to set.
	 */
	public void setSizeProportions(float widthPercent, float heightPercent) {
		this.widthPercent = widthPercent;
		this.heightPercent = heightPercent;
	}

	@Override
	public void apply(Rectangle contentBounds, Rectangle sizeHint) {
		int widthHint = sizeHint.getWidth();
		if (this.widthPercent != -1 && widthHint != MWT.NONE) {
			contentBounds.setWidth((int) (widthHint * this.widthPercent));
		}
		int heightHint = sizeHint.getHeight();
		if (this.heightPercent != -1 && heightHint != MWT.NONE) {
			contentBounds.setHeight((int) (heightHint * this.heightPercent));
		}
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof RelativeDimension) {
			RelativeDimension dimension = (RelativeDimension) obj;
			return this.widthPercent == dimension.widthPercent && this.heightPercent == dimension.heightPercent;
		}
		return false;
	}

	@Override
	public int hashCode() {
		// Since the fields are not final, they can't be used to compute a constant hashcode.
		return 0;
	}

}
