/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.outline;

import ej.annotation.Nullable;
import ej.microui.display.GraphicsContext;
import ej.style.container.Rectangle;

/**
 * Complex outline that has a different thickness for each edge.
 */
public class ComplexOutline implements Outline {

	private int top;
	private int bottom;
	private int left;
	private int right;

	/**
	 * Creates an empty complex outline (0 pixel thickness).
	 */
	public ComplexOutline() {
	}

	/**
	 * Creates a complex outline specifying its thickness for each edge.
	 *
	 * @param top
	 *            the top thickness to set.
	 * @param right
	 *            the right thickness to set.
	 * @param bottom
	 *            the bottom thickness to set.
	 * @param left
	 *            the left thickness to set.
	 */
	public ComplexOutline(int top, int right, int bottom, int left) {
		this.top = top;
		this.right = right;
		this.bottom = bottom;
		this.left = left;
	}

	/**
	 * Gets the top thickness.
	 *
	 * @return the top thickness.
	 */
	public int getTop() {
		return this.top;
	}

	/**
	 * Sets the top thickness.
	 *
	 * @param thickness
	 *            the thickness to set.
	 */
	public void setTop(int thickness) {
		this.top = thickness;
	}

	/**
	 * Gets the bottom thickness.
	 *
	 * @return the bottom thickness.
	 */
	public int getBottom() {
		return this.bottom;
	}

	/**
	 * Sets the bottom thickness.
	 *
	 * @param thickness
	 *            the thickness to set.
	 */
	public void setBottom(int thickness) {
		this.bottom = thickness;
	}

	/**
	 * Gets the left thickness.
	 *
	 * @return the left thickness.
	 */
	public int getLeft() {
		return this.left;
	}

	/**
	 * Sets the left thickness.
	 *
	 * @param thickness
	 *            the thickness to set.
	 */
	public void setLeft(int thickness) {
		this.left = thickness;
	}

	/**
	 * Gets the right thickness.
	 *
	 * @return the right thickness.
	 */
	public int getRight() {
		return this.right;
	}

	/**
	 * Sets the right thickness.
	 *
	 * @param thickness
	 *            the thickness to set.
	 */
	public void setRight(int thickness) {
		this.right = thickness;
	}

	@Override
	public void wrap(Rectangle rectangle) {
		rectangle.update(-this.left, -this.top, this.left + this.right, this.top + this.bottom);
	}

	@Override
	public void unwrap(Rectangle rectangle) {
		rectangle.update(this.left, this.top, -(this.left + this.right), -(this.top + this.bottom));
	}

	@Override
	public void apply(GraphicsContext g, Rectangle rectangle) {
		g.translate(this.left, this.top);
		unwrap(rectangle);
		g.clipRect(0, 0, rectangle.getWidth(), rectangle.getHeight());
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof ComplexOutline) {
			ComplexOutline outline = (ComplexOutline) obj;
			return this.top == outline.top && this.bottom == outline.bottom && this.left == outline.left
					&& this.right == outline.right;
		}
		return false;
	}

	@Override
	public int hashCode() {
		// Since the fields are not final, they can't be used to compute a constant hashcode.
		return 0;
	}

}
