/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector;

import ej.annotation.Nullable;
import ej.style.Element;
import ej.style.Selector;

/**
 * A nth child selector selects by checking if an element is the nth child of its parent.
 * <p>
 * Equivalent to <code>:nth-child</code> selector in CSS. Its specificity is (0,0,1,0).
 *
 * @see Element#getParentElement()
 * @see SpecificityHelper
 */
public class NthChildSelector implements Selector {

	private final int index;

	/**
	 * Creates a nth child selector.
	 *
	 * @param index
	 *            the index of the child.
	 */
	public NthChildSelector(int index) {
		this.index = index;
	}

	@Override
	public boolean fit(Element element) {
		Element parentElement = element.getParentElement();

		return parentElement != null && parentElement.getChildrenCount() > this.index
				&& parentElement.getChild(this.index) == element;
	}

	@Override
	public int getSpecificity() {
		return SpecificityHelper.getSpecificity(0, 0, 1, 0);
	}

	@Override
	public int hashCode() {
		return this.index;
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof NthChildSelector) {
			return ((NthChildSelector) obj).index == this.index;
		}
		return false;
	}

}
