/*
 * Java
 *
 * Copyright 2018-2019 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.util.firmwareupdate;

import java.io.IOException;

/**
 * This interface provides methods to program a new firmware on the device.
 */
public interface FirmwareProgrammer {

	/**
	 * Initializes a firmware update procedure.
	 *
	 * This method should be used when the size of the firmware is not known yet, otherwise the method
	 * {@link #initialize(int)} should be used instead.
	 *
	 * @throws IOException
	 *             if an I/O error occurred
	 */
	public void initialize() throws IOException;

	/**
	 * Initializes a firmware update procedure.
	 *
	 * This method should be used when the size of the firmware is already known, otherwise the method
	 * {@link #initialize()} should be used instead.
	 *
	 * @param firmwareSize
	 *            the size of the firmware data
	 * @throws IOException
	 *             if an I/O error occurred
	 * @throws IllegalArgumentException
	 *             if firmwareSize is zero or a negative integer
	 */
	public void initialize(int firmwareSize) throws IOException;

	/**
	 * Writes a chunk of firmware data as part of the firmware update.
	 *
	 * @param data
	 *            the firmware data chunk
	 * @param offset
	 *            the offset in the array
	 * @param length
	 *            the number of bytes to write
	 * @throws IOException
	 *             if an I/O error occurred
	 */
	public void write(byte[] data, int offset, int length) throws IOException;

	/**
	 * Finishes the firmware update procedure.
	 *
	 * @throws IOException
	 *             if an I/O error occurred
	 */
	public void finish() throws IOException;

	/**
	 * Resets the device.
	 *
	 * @throws IOException
	 *             if an I/O error occurred
	 */
	public void reset() throws IOException;
}
