/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.util.map;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * A weak value hash map is a map that store that keep only weak references on the values it contains.
 *
 * @param <K>
 *            the type of keys maintained by this map
 * @param <V>
 *            the type of mapped values
 * @see HashMap
 * @see WeakReference
 */
public class WeakValueHashMap<K, V> implements Map<K, V> {

	/**
	 * Default initial map capacity.
	 */
	public static final int INITIAL_CAPACITY = 16;
	private final Map<K, WeakValue> map;
	private final ReferenceQueue<V> queue;

	private class WeakValue extends WeakReference<V> {
		K key;

		public WeakValue(K key, V referent, ReferenceQueue<V> queue) {
			super(referent, queue);
			this.key = key;
		}
	}

	/**
	 * Constructs an empty map with the default initial capacity ({@value #INITIAL_CAPACITY}) and the default load
	 * factor (0.75).
	 */
	public WeakValueHashMap() {
		this(INITIAL_CAPACITY);
	}

	/**
	 * Constructs an empty map with the specified initial capacity and the default load factor (0.75).
	 *
	 * @param initialCapacity
	 *            the initial capacity.
	 * @throws IllegalArgumentException
	 *             if the initial capacity is negative.
	 */
	public WeakValueHashMap(int initialCapacity) {
		this.map = new HashMap<>(initialCapacity);
		this.queue = new ReferenceQueue<>();
	}

	private void cleanup() {
		while (true) {
			@SuppressWarnings("unchecked")
			WeakValue value = (WeakValue) this.queue.poll();
			if (value != null) {
				this.map.remove(value.key);
			} else {
				break;
			}
		}
	}

	private V getValue(WeakValue weakValue) {
		return weakValue == null ? null : weakValue.get();
	}

	@Override
	public void clear() {
		this.map.clear();
	}

	@Override
	public boolean containsKey(Object key) {
		return this.map.containsKey(key);
	}

	@Override
	public boolean containsValue(Object value) {
		// Cleanup the map before.
		cleanup();
		Collection<WeakValueHashMap<K, V>.WeakValue> values = this.map.values();
		for (WeakValue weakValue : values) {
			if (value == getValue(weakValue)) {
				return true;
			}
		}
		return false;
	}

	@Override
	public Set<Map.Entry<K, V>> entrySet() {
		// Cleanup the map before.
		cleanup();
		Set<Map.Entry<K, V>> result = new HashSet<Map.Entry<K, V>>();
		for (Map.Entry<K, WeakValueHashMap<K, V>.WeakValue> entry : this.map.entrySet()) {
			result.add(new AbstractMap.SimpleEntry<K, V>(entry.getKey(), getValue(entry.getValue())));
		}
		return result;
	}

	@Override
	public V get(Object key) {
		return getValue(this.map.get(key));
	}

	@Override
	public boolean isEmpty() {
		// Cleanup the map before.
		cleanup();
		return this.map.isEmpty();
	}

	@Override
	public Set<K> keySet() {
		// Cleanup the map before.
		cleanup();
		return this.map.keySet();
	}

	@Override
	public V put(K key, V value) {
		cleanup();
		WeakValue weakValue = new WeakValue(key, value, this.queue);
		return getValue(this.map.put(key, weakValue));
	}

	@Override
	public void putAll(Map<? extends K, ? extends V> m) {
		for (Map.Entry<? extends K, ? extends V> e : m.entrySet()) {
			put(e.getKey(), e.getValue());
		}
	}

	@Override
	public V remove(Object key) {
		return getValue(this.map.remove(key));
	}

	@Override
	public int size() {
		// Cleanup the map before.
		cleanup();
		return this.map.size();
	}

	@Override
	public Collection<V> values() {
		// Cleanup the map before.
		cleanup();
		Collection<V> result = new ArrayList<V>(size());
		for (WeakValue weakValue : this.map.values()) {
			result.add(getValue(weakValue));
		}
		return result;
	}

}
