/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget;

import ej.annotation.Nullable;
import ej.basictool.ArrayTools;
import ej.microui.display.GraphicsContext;
import ej.mwt.Widget;
import ej.style.Element;
import ej.style.State;
import ej.style.Style;
import ej.style.background.Background;
import ej.style.container.Rectangle;
import ej.style.selector.InstanceSelector;
import ej.style.util.StyleHelper;

/**
 * A widget using a style specified in a style sheet. Especially the style sheet defines a margin, a border and a
 * padding for each widget.
 */
public abstract class StyledWidget extends Widget implements Element, StyledRenderable {

	// Some (most) of the code in this class is duplicated in all StyledRenderable implementors for footprint
	// optimization (flash & RAM).

	/**
	 * The cached transparent state.
	 */
	private boolean transparent;
	/**
	 * The cached style.
	 */
	@Nullable
	private Style style;
	/**
	 * The class selectors.
	 */
	private String[] classSelectors;

	/**
	 * Creates a styled widget without class selector.
	 */
	protected StyledWidget() {
		this.classSelectors = StyledHelper.EMPTY_STRING_ARRAY;
	}

	@Override
	public boolean isTransparent() {
		return this.transparent;
	}

	@Override
	public boolean contains(int x, int y) {
		return StyledHelper.contains(getX(), getY(), getWidth(), getHeight(), this.style, x, y);
	}

	@Override
	@Nullable
	public Element getParentElement() {
		return StyledHelper.getParentElement(this);
	}

	@Override
	public Element[] getChildrenElements() {
		return EMPTY_ELEMENT_ARRAY;
	}

	@Override
	public Element getChild(int index) {
		throw new IndexOutOfBoundsException();
	}

	@Override
	public int getChildrenCount() {
		return 0;
	}

	@Override
	public void mergeStyle(Style style) {
		StyleHelper.getStylesheet().addRule(new InstanceSelector(this), style);
		updateStyle();
	}

	@Override
	public void removeInstanceStyle() {
		StyleHelper.getStylesheet().removeRule(new InstanceSelector(this));
		updateStyle();
	}

	@Override
	public Style getStyle() {
		Style style = this.style;
		if (style == null) {
			style = StyledHelper.getStyleFromStylesheet(this);
			setStyle(style);
		}
		return style;
	}

	private void setStyle(Style newStyle) {
		this.style = newStyle;
		Background background = newStyle.getBackground();
		this.transparent = background.isTransparent();
	}

	/**
	 * Updates the style of the widget.
	 */
	protected void updateStyle() {
		// Do not update style if not shown. See validate method.
		if (isShown() && updateStyleOnly()) {
			repaint();
		}
	}

	/**
	 * Update widget style without repainting it.
	 *
	 * @return <code>true</code> if the style has been updated, <code>false</code> otherwise.
	 */
	protected boolean updateStyleOnly() {
		Style newStyle = StyledHelper.getStyleFromStylesheet(this);

		if (!newStyle.equals(this.style)) {
			setStyle(newStyle);
			return true;
		}
		return false;
	}

	/**
	 * Gets the content bounds of this widget. That means the size minus the different outlines.
	 *
	 * @return the content bounds.
	 * @see #getWidth()
	 * @see #getHeight()
	 * @see StyleHelper#computeContentBounds(Rectangle, Style)
	 */
	protected Rectangle getContentBounds() {
		Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
		return StyleHelper.computeContentBounds(bounds, getStyle());
	}

	@Override
	public void render(GraphicsContext g) {
		StyledHelper.render(g, this);
	}

	@Override
	public void validate(int widthHint, int heightHint) {
		if (isVisible()) {
			setStyle(StyledHelper.getStyleFromStylesheet(this));
			Rectangle bounds = StyledHelper.validate(widthHint, heightHint, this);
			setPreferredSize(bounds.getWidth(), bounds.getHeight());
		} else {
			setPreferredSize(0, 0);
		}
	}

	@Override
	public void setEnabled(boolean enabled) {
		if (isEnabled() != enabled) {
			super.setEnabled(enabled);
			updateStyle();
		}
	}

	@Override
	public boolean hasClassSelector(String classSelector) {
		return ArrayTools.containsEquals(this.classSelectors, classSelector);
	}

	@Override
	public void addClassSelector(String classSelector) {
		this.classSelectors = StyledHelper.addClassSelector(this.classSelectors, classSelector);
		updateStyle();
	}

	@Override
	public void removeClassSelector(String classSelector) {
		int classSelectorsLength = this.classSelectors.length;
		this.classSelectors = ArrayTools.remove(this.classSelectors, classSelector);
		if (this.classSelectors.length != classSelectorsLength) {
			updateStyle();
		}
	}

	@Override
	public void setClassSelectors(String classSelector) {
		this.classSelectors = StyledHelper.setClassSelectors(classSelector);
		updateStyle();
	}

	@Override
	public void removeAllClassSelectors() {
		if (this.classSelectors.length != 0) {
			this.classSelectors = StyledHelper.EMPTY_STRING_ARRAY;
			updateStyle();
		}
	}

	@Override
	public void gainFocus() {
		super.gainFocus();
		updateStyle();
	}

	@Override
	public void lostFocus() {
		super.lostFocus();
		updateStyle();
	}

	@Override
	public boolean isInState(State state) {
		return (state == State.Enabled && isEnabled()) || (state == State.Disabled && !isEnabled())
				|| (state == State.Focus && hasFocus());
	}

	@Override
	@Nullable
	public String getAttribute(String attribute) {
		return null;
	}

}
