/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic;

import ej.giml.annotation.Element;
import ej.giml.annotation.ElementConstructor;
import ej.giml.annotation.ElementRequiredAttribute;
import ej.style.State;
import ej.widget.listener.OnClickListener;
import ej.widget.util.ButtonHelper;
import ej.widget.util.GenericListener;

/**
 * A button is a widget that displays a text and reacts to click events.
 * <p>
 * This example shows a simple button:
 *
 * <pre>
 * Button button = new Button("Press Me");
 * </pre>
 *
 * <img src="doc-files/button-simple.png" alt="Simple button.">
 * <p>
 * This example shows a styled button:
 *
 * <pre>
 * Button button = new Button("Press Me");
 *
 * EditableStyle buttonStyle = new EditableStyle();
 * buttonStyle.setBackgroundColor(Colors.SILVER);
 * buttonStyle.setBorder(new SimpleRectangularBorder(1));
 * buttonStyle.setFontProfile(new FontProfile("", 23, Font.STYLE_PLAIN));
 * button.mergeStyle(buttonStyle);
 * </pre>
 *
 * <img src="doc-files/button-styled.png" alt="Styled button.">
 * <p>
 * This example shows a styled button with its background color changing on a press action:
 *
 * <pre>
 * Button button = new Button("Press Me");
 *
 * Stylesheet stylesheet = StyleHelper.getStylesheet();
 *
 * EditableStyle buttonStyle = new EditableStyle();
 * buttonStyle.setBackgroundColor(Colors.SILVER);
 * buttonStyle.setBorder(new SimpleRectangularBorder(1));
 * buttonStyle.setFontProfile(new FontProfile("", 23, Font.STYLE_PLAIN));
 * stylesheet.addRule(new TypeSelector(Button.class), buttonStyle);
 *
 * EditableStyle pressedButtonStyle = new EditableStyle();
 * pressedButtonStyle.setBackgroundColor(Colors.GRAY);
 * stylesheet.addRule(new AndCombinator(new TypeSelector(Button.class), new StateSelector(State.Active)),
 * 		pressedButtonStyle);
 * </pre>
 *
 * <img src="doc-files/button-styled-pressed.png" alt="Styled pressed button.">
 * 
 * @since 2.3.0
 */
@Element
public class Button extends Label {

	private final ButtonHelper buttonHelper;

	/**
	 * Creates a button with an empty text.
	 */
	public Button() {
		this(""); //$NON-NLS-1$
	}

	/**
	 * Creates a button with the given text to display. The text cannot be <code>null</code>.
	 *
	 * @param text
	 *            the text to display.
	 * @throws NullPointerException
	 *             if the text is <code>null</code>.
	 */
	@ElementConstructor
	public Button(@ElementRequiredAttribute String text) {
		super(text);
		setEnabled(true);
		this.buttonHelper = new ButtonHelper(new GenericListener() {
			@Override
			public void update() {
				updateStyle();
			}
		});
		this.buttonHelper.addOnClickListener(new OnClickListener() {
			@Override
			public void onClick() {
				Button.this.onClick();
			}
		});
	}

	/**
	 * Invoked when the button has been clicked.
	 */
	protected void onClick() {
		// Do nothing by default.
	}

	/**
	 * Performs the actions associated to a click.
	 */
	public void performClick() {
		this.buttonHelper.performClick();
	}

	/**
	 * Adds a listener on the click events of the button.
	 *
	 * @param listener
	 *            the listener to add.
	 * @throws NullPointerException
	 *             if the given listener is <code>null</code>.
	 */
	public void addOnClickListener(OnClickListener listener) {
		this.buttonHelper.addOnClickListener(listener);
	}

	/**
	 * Removes a listener on the click events of the button.
	 *
	 * @param listener
	 *            the listener to remove.
	 */
	public void removeOnClickListener(OnClickListener listener) {
		this.buttonHelper.removeOnClickListener(listener);
	}

	@Override
	public boolean isInState(State state) {
		return (state == State.Active && this.buttonHelper.isPressed()) || super.isInState(state);
	}

	@Override
	public boolean handleEvent(int event) {
		if (this.buttonHelper.handleEvent(event)) {
			return true;
		}
		return super.handleEvent(event);
	}

}
