/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.composed;

import ej.giml.annotation.ElementChild;
import ej.mwt.MWT;
import ej.mwt.Widget;
import ej.style.State;
import ej.style.Style;
import ej.style.container.AlignmentHelper;
import ej.style.container.Rectangle;
import ej.widget.basic.Box;
import ej.widget.basic.Label;
import ej.widget.toggle.ToggleModel;

/**
 * A facility to use a toggle that displays a text.
 * <p>
 * It is simply a toggle wrapper that contains a label, based on a toggle model.
 *
 * @see Box
 * @see Label
 * @see ToggleModel
 */
public class Toggle extends ToggleWrapper {

	private final Box box;
	private final Label label;

	/**
	 * Creates a toggle with the given text to display. The text cannot be <code>null</code>.
	 * <p>
	 * The box state is updated along with the state of the toggle.
	 *
	 * @param box
	 *            the widget representing the state of the toggle.
	 * @param text
	 *            the text to display.
	 * @throws NullPointerException
	 *             if a parameter is <code>null</code>.
	 * @see State#Checked
	 */
	public Toggle(Box box, String text) {
		this(new ToggleModel(), box, text);
	}

	/**
	 * Creates a toggle with the given text to display. The text cannot be <code>null</code>.
	 * <p>
	 * The box state is updated along with the state of the toggle.
	 *
	 * @param toggleModel
	 *            the toggle model.
	 * @param box
	 *            the widget representing the state of the toggle.
	 * @param text
	 *            the text to display.
	 * @throws NullPointerException
	 *             if a parameter is <code>null</code>.
	 * @see State#Checked
	 */
	public Toggle(ToggleModel toggleModel, Box box, String text) {
		super(toggleModel);
		this.box = box;
		this.label = new Label(text);
		addWidgets(box);
	}

	/**
	 * Creates a toggle with the given text to display. The text cannot be <code>null</code>.
	 * <p>
	 * The box state is updated along with the state of the toggle.
	 *
	 * @param toggleModel
	 *            the toggle model.
	 * @param box
	 *            the widget representing the state of the toggle.
	 * @param text
	 *            the text to display.
	 * @param group
	 *            the name of the toggle group.
	 * @throws NullPointerException
	 *             if a parameter is <code>null</code>.
	 * @see State#Checked
	 */
	public Toggle(ToggleModel toggleModel, Box box, String text, String group) {
		super(toggleModel, group);
		this.box = box;
		this.label = new Label(text);
		addWidgets(box);
	}

	private void addWidgets(Box box) {
		add(box);
		add(this.label);
	}

	@Override
	@ElementChild(prefix = "setWidget", disabled = true)
	public void setWidget(Widget widget) {
		throw new IllegalArgumentException();
	}

	/**
	 * Gets the label contained in the toggle.
	 *
	 * @return the label contained in the toggle.
	 */
	public Label getLabel() {
		return this.label;
	}

	/**
	 * Sets the text displayed by this toggle.
	 *
	 * @param text
	 *            the text to display.
	 * @throws NullPointerException
	 *             if the text is <code>null</code>.
	 * @see Label#setText(String)
	 */
	public void setText(String text) {
		this.label.setText(text);
	}

	/**
	 * Gets the text displayed by this toggle.
	 * <p>
	 * The text is hold by the inner label.
	 *
	 * @return the text.
	 * @see Label#getText()
	 */
	public String getText() {
		return this.label.getText();
	}

	@Override
	public void update() {
		Box box = this.box;
		if (box != null) {
			box.setChecked(isChecked());
		}
	}

	@Override
	protected void setPressed(boolean pressed) {
		super.setPressed(pressed);
		Box box = this.box;
		if (box != null) {
			box.setPressed(pressed);
		}
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		int widthHint = bounds.getWidth();
		int heightHint = bounds.getHeight();

		boolean computeWidth = widthHint == MWT.NONE;
		boolean computeHeight = heightHint == MWT.NONE;

		Box box = this.box;
		Label label = this.label;

		int availableHeight;
		if (isAdjustedToChild()) {
			availableHeight = heightHint;
		} else {
			availableHeight = MWT.NONE;
		}

		// validate widgets and get their preferred widgets
		box.validate(MWT.NONE, availableHeight);
		int boxPreferredWidth = box.getPreferredWidth();

		// label take remaining size
		int labelWidth;
		if (!computeWidth) {
			labelWidth = Math.max(0, widthHint - boxPreferredWidth);
		} else {
			labelWidth = MWT.NONE;
		}
		label.validate(labelWidth, availableHeight);

		// compute composite preferred size if necessary
		if (computeWidth) {
			widthHint = boxPreferredWidth + label.getPreferredWidth();
		}
		if (computeHeight) {
			heightHint = Math.max(box.getPreferredHeight(), label.getPreferredHeight());
		}

		return new Rectangle(0, 0, widthHint, heightHint);
	}

	@Override
	protected void setBoundsContent(Rectangle bounds) {
		int boundsX = bounds.getX();
		int boundsY = bounds.getY();
		int boundsWidth = bounds.getWidth();
		int boundsHeight = bounds.getHeight();

		int boxWidth = this.box.getPreferredWidth();

		// Set widgets bounds.
		if (isAdjustedToChild()) {
			this.box.setBounds(boundsX, boundsY, boxWidth, boundsHeight);
			this.label.setBounds(boundsX + boxWidth, boundsY, Math.max(0, boundsWidth - boxWidth), boundsHeight);
		} else {
			int boxHeight = this.box.getPreferredHeight();
			int labelWidth = this.label.getPreferredWidth();
			int labelHeight = this.label.getPreferredHeight();
			int alignment = getStyle().getAlignment();
			int x = AlignmentHelper.computeXLeftCorner(boxWidth + labelWidth, boundsX, boundsWidth, alignment);
			int boxY = AlignmentHelper.computeYTopCorner(boxHeight, boundsY, boundsHeight, alignment);
			int labelY = AlignmentHelper.computeYTopCorner(labelHeight, boundsY, boundsHeight, alignment);

			this.box.setBounds(x, boxY, boxWidth, boxHeight);
			this.label.setBounds(x + boxWidth, labelY, labelWidth, labelHeight);
		}
	}

}
