/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ej.annotation.Nullable;
import ej.giml.annotation.Element;
import ej.giml.annotation.ElementChild;
import ej.mwt.MWT;
import ej.mwt.Widget;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.widget.StyledComposite;

/**
 * Docks its children to the left, right, top or bottom sides.
 * <p>
 * Each child can be pushed towards any direction. One child can be set to the center.
 * <p>
 * The dock manages multiple widgets per side.
 * <p>
 * Following the addition order, each widget is pushed against the side of the remaining area. The remaining area is
 * decreased by each children size (width for left and right, height for top and bottom). The center widget (if any)
 * fills the remaining area.
 * <p>
 * Example of dock that contains (added in this order):
 * <ol>
 * <li>widget on the left,</li>
 * <li>widget on the top,</li>
 * <li>widget on the right,</li>
 * <li>widget on the bottom,</li>
 * <li>widget on the left,</li>
 * <li>widget on the bottom,</li>
 * <li>widget on the center.</li>
 * </ol>
 * <img src="doc-files/dock.png" alt="Dock.">
 */
@Element
public class Dock extends StyledComposite {

	private final List<WidgetConstraint> widgetConstraints;
	@Nullable
	private Widget center;

	/**
	 * Creates a dock composite.
	 */
	public Dock() {
		this.widgetConstraints = new ArrayList<>(2);
	}

	/**
	 * Docks a widget to the left of this composite.
	 *
	 * @param widget
	 *            the widget to add.
	 * @throws NullPointerException
	 *             if the given widget is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given widget or one of its children is already connected to a panel.
	 * @see #add(Widget)
	 */
	@ElementChild
	public void addLeft(Widget widget) {
		this.widgetConstraints.add(new WidgetConstraint(widget, MWT.LEFT));
		add(widget);
	}

	/**
	 * Docks a widget to the right of this composite.
	 *
	 * @param widget
	 *            the widget to add.
	 * @throws NullPointerException
	 *             if the given widget is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given widget or one of its children is already connected to a panel.
	 * @see #add(Widget)
	 */
	@ElementChild
	public void addRight(Widget widget) {
		this.widgetConstraints.add(new WidgetConstraint(widget, MWT.RIGHT));
		add(widget);
	}

	/**
	 * Docks a widget to the top of this composite.
	 *
	 * @param widget
	 *            the widget to add.
	 * @throws NullPointerException
	 *             if the given widget is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given widget or one of its children is already connected to a panel.
	 * @see #add(Widget)
	 */
	@ElementChild
	public void addTop(Widget widget) {
		this.widgetConstraints.add(new WidgetConstraint(widget, MWT.TOP));
		add(widget);
	}

	/**
	 * Docks a widget to the bottom of this composite.
	 *
	 * @param widget
	 *            the widget to add.
	 * @throws NullPointerException
	 *             if the given widget is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given widget or one of its children is already connected to a panel.
	 * @see #add(Widget)
	 */
	@ElementChild
	public void addBottom(Widget widget) {
		this.widgetConstraints.add(new WidgetConstraint(widget, MWT.BOTTOM));
		add(widget);
	}

	/**
	 * Sets the center widget of this composite.
	 *
	 * @param widget
	 *            the widget to add.
	 * @throws NullPointerException
	 *             if the given widget is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given widget or one of its children is already connected to a panel.
	 * @see #add(Widget)
	 */
	@ElementChild
	public void setCenter(Widget widget) {
		Widget centerLocal = this.center; // avoid synchronizing
		if (centerLocal != null) {
			remove(centerLocal);
		}
		this.center = widget;
		add(widget);
	}

	@Override
	public void remove(Widget widget) {
		if (widget == this.center) {
			this.center = null;
		} else {
			Iterator<WidgetConstraint> iterator = this.widgetConstraints.iterator();
			while (iterator.hasNext()) {
				WidgetConstraint widgetConstraint = iterator.next();
				if (widgetConstraint.widget == widget) {
					iterator.remove();
					break;
				}
			}
		}
		super.remove(widget);
	}

	/**
	 * Removes the center widget of this dock.
	 *
	 * @see #remove(Widget)
	 */
	public void removeCenter() {
		Widget center = this.center;
		if (center != null) {
			remove(center);
			this.center = null;
		}
	}

	@Override
	public void removeAllWidgets() {
		super.removeAllWidgets();
		this.widgetConstraints.clear();
		this.center = null;
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		int widthHint = bounds.getWidth();
		int heightHint = bounds.getHeight();

		boolean computeWidth = widthHint == MWT.NONE;
		boolean computeHeight = heightHint == MWT.NONE;

		int remainingWidth = widthHint;
		int remainingHeight = heightHint;

		// Lay out docked widgets.
		List<WidgetConstraint> widgetConstraints = this.widgetConstraints;
		int widgetConstraintsLength = widgetConstraints.size();
		for (int i = -1; ++i < widgetConstraintsLength;) {
			WidgetConstraint widgetConstraint = widgetConstraints.get(i);
			Widget widget = widgetConstraint.widget;
			int position = widgetConstraint.position;

			// Compute widget size hints.
			int widgetWidthHint;
			int widgetHeightHint;
			switch (position) {
			case MWT.LEFT:
			case MWT.RIGHT:
				widgetWidthHint = MWT.NONE;
				widgetHeightHint = remainingHeight;
				break;
			case MWT.TOP:
			case MWT.BOTTOM:
				widgetWidthHint = remainingWidth;
				widgetHeightHint = MWT.NONE;
				break;
			default:
				// Sanity check (& for the compiler).
				throw new IllegalStateException();
			}
			// Validate widget.
			widget.validate(widgetWidthHint, widgetHeightHint);

			// Decrease remaining space.
			switch (position) {
			case MWT.LEFT:
			case MWT.RIGHT:
				if (!computeWidth) {
					remainingWidth -= widget.getPreferredWidth();
				}
				break;
			case MWT.TOP:
			case MWT.BOTTOM:
				if (!computeHeight) {
					remainingHeight -= widget.getPreferredHeight();
				}
				break;
			}
		}
		// Lay out center.
		Widget center = this.center;
		if (center != null) {
			center.validate(remainingWidth, remainingHeight);
		}

		// Compute preferred size if necessary.
		if (computeWidth || computeHeight) {
			int totalWidth;
			int totalHeight;

			if (center != null) {
				totalWidth = center.getPreferredWidth();
				totalHeight = center.getPreferredHeight();
			} else {
				totalWidth = 0;
				totalHeight = 0;
			}

			// Browse by the end to compute the total size.
			for (int i = widgetConstraintsLength; --i >= 0;) {
				WidgetConstraint widgetConstraint = widgetConstraints.get(i);
				Widget widget = widgetConstraint.widget;
				int widgetPreferredWidth = widget.getPreferredWidth();
				int widgetPreferredHeight = widget.getPreferredHeight();

				switch (widgetConstraint.position) {
				case MWT.LEFT:
				case MWT.RIGHT:
					totalWidth += widgetPreferredWidth;
					totalHeight = Math.max(totalHeight, widgetPreferredHeight);
					break;
				case MWT.TOP:
				case MWT.BOTTOM:
					totalWidth = Math.max(totalWidth, widgetPreferredWidth);
					totalHeight += widgetPreferredHeight;
					break;
				}
			}

			if (computeWidth) {
				widthHint = totalWidth;
			}
			if (computeHeight) {
				heightHint = totalHeight;
			}
		}

		return new Rectangle(0, 0, widthHint, heightHint);
	}

	@Override
	protected void setBoundsContent(Rectangle bounds) {
		int x = bounds.getX();
		int y = bounds.getY();
		int width = bounds.getWidth();
		int height = bounds.getHeight();

		// Lay out docked widgets.
		List<WidgetConstraint> widgetConstraints = this.widgetConstraints;
		int widgetConstraintsLength = widgetConstraints.size();
		for (int i = -1; ++i < widgetConstraintsLength;) {
			WidgetConstraint widgetConstraint = widgetConstraints.get(i);
			Widget widget = widgetConstraint.widget;
			int position = widgetConstraint.position;

			int widgetPreferredWidth = widget.getPreferredWidth();
			int widgetPreferredHeight = widget.getPreferredHeight();

			switch (position) {
			case MWT.LEFT:
				widget.setBounds(x, y, widgetPreferredWidth, height);
				x += widgetPreferredWidth;
				width -= widgetPreferredWidth;
				break;
			case MWT.RIGHT:
				widget.setBounds(x + width - widgetPreferredWidth, y, widgetPreferredWidth, height);
				width -= widgetPreferredWidth;
				break;
			case MWT.TOP:
				widget.setBounds(x, y, width, widgetPreferredHeight);
				y += widgetPreferredHeight;
				height -= widgetPreferredHeight;
				break;
			case MWT.BOTTOM:
				widget.setBounds(x, y + height - widgetPreferredHeight, width, widgetPreferredHeight);
				height -= widgetPreferredHeight;
				break;
			}
		}
		// Lay out center.
		Widget center = this.center;
		if (center != null) {
			center.setBounds(x, y, width, height);
		}
	}

}

class WidgetConstraint {
	Widget widget;
	/**
	 * One of {@link MWT#LEFT}, {@link MWT#RIGHT}, {@link MWT#TOP}, {@link MWT#BOTTOM}.
	 */
	int position;

	WidgetConstraint(Widget widget, int position) {
		this.widget = widget;
		this.position = position;
	}
}
