/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container.transition;

import ej.annotation.Nullable;
import ej.microui.display.GraphicsContext;
import ej.microui.display.Image;
import ej.mwt.Composite;
import ej.mwt.Panel;
import ej.mwt.Widget;
import ej.style.Style;
import ej.style.background.Background;
import ej.style.container.Rectangle;
import ej.style.util.StyleHelper;
import ej.widget.util.DrawScreenHelper;

/**
 * A screenshot transition container uses a screenshot to realize the animation.
 *
 * @since 2.3.0
 */
public abstract class ScreenshotTransitionContainer extends TransitionContainer {

	/**
	 * <code>true</code> if going forward, <code>false</code> otherwise.
	 *
	 * @see #show(Widget, boolean)
	 */
	protected boolean forward;
	/**
	 * The new widget to show.
	 *
	 * @see #show(Widget, boolean)
	 */
	@Nullable
	protected Widget newWidget;
	/**
	 * The screenshot of the new widget.
	 */
	@Nullable
	protected Image newScreenshot;

	/**
	 * The stop value of the motion.
	 *
	 * @see #getStop(int, int)
	 */
	protected int stop;
	/**
	 * The current step value of the motion.
	 */
	protected int step;

	private boolean isAnimating;

	@Override
	public boolean isTransparent() {
		if (this.isAnimating) {
			// During the transition, the container cannot be transparent as long as the content of the display is used
			// for the rendering. Its parent must not clean the screen in this area.
			return false;
		} else {
			return super.isTransparent();
		}
	}

	@Override
	public void show(Widget widget, boolean forward) {
		// /!\ Beware that if some parameters can be changed between animation, the context of the previous
		// animation must be saved in order to finalize it correctly!
		cancelAnimation();

		// Update context after cleaning the previous one.
		this.forward = forward;

		Panel panel = getPanel();
		boolean shown = panel != null && panel.isShown();
		int widgetsCount = getWidgetsCount();
		if (!shown || widgetsCount == 0) {
			// It may not be shown, but have widgets, remove them first.
			removeAllWidgets();

			add(widget);
			revalidateSubTree();
		} else {
			if (widget.getPanel() != null || widget.getParent() != null) {
				throw new IllegalArgumentException();
			}
			// Save new widget only for animation.
			this.newWidget = widget;

			Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
			Rectangle contentBounds = StyleHelper.computeContentBounds(bounds, getStyle());

			// Add the widget in the hierarchy to inherit the style of the parent (this).
			// The widget is removed shortly after along the current one.
			add(widget);
			takeScreenshot(widget, contentBounds);

			Widget previousWidget = getWidget(0);
			removeAllWidgets();

			this.stop = getStop(contentBounds.getWidth(), contentBounds.getHeight());
			this.isAnimating = true;
			startAnimation(widget, previousWidget, forward, 0, this.stop);
		}
	}

	/**
	 * Takes a screenshot of a widget.
	 *
	 * @param widget
	 *            the widget.
	 * @param contentBounds
	 *            the content bounds.
	 */
	protected void takeScreenshot(Widget widget, Rectangle contentBounds) {
		int contentX = contentBounds.getX();
		int contentY = contentBounds.getY();
		int contentWidth = contentBounds.getWidth();
		int contentHeight = contentBounds.getHeight();

		Image newScreenshot = Image.createImage(contentWidth, contentHeight);
		GraphicsContext screenshotGraphicsContext = newScreenshot.getGraphicsContext();

		// Prepare widget.
		widget.validate(contentWidth, contentHeight);
		widget.setBounds(contentX, contentY, contentWidth, contentHeight);

		// Draw this background and content.
		Style style = getStyle();
		Background background = style.getBackground();
		background.apply(screenshotGraphicsContext, contentBounds, style.getBackgroundColor());
		renderContent(screenshotGraphicsContext, style, contentBounds);

		// Draw the widget.
		if (widget instanceof Composite) {
			DrawScreenHelper.draw(screenshotGraphicsContext, (Composite) widget);
		} else {
			DrawScreenHelper.draw(screenshotGraphicsContext, widget);
		}

		this.newScreenshot = newScreenshot;
	}

	/**
	 * Gets the stop value of the motion. The start value being <code>0</code>.
	 *
	 * @param contentWidth
	 *            the content width.
	 * @param contentHeight
	 *            the content height.
	 * @return the stop value of the motion.
	 */
	protected abstract int getStop(int contentWidth, int contentHeight);

	/**
	 * Gets whether the transition container is doing the transition or not.
	 *
	 * @return <code>true</code> during the animation, <code>false</code> otherwise.
	 */
	protected boolean isAnimating() {
		return this.isAnimating;
	}

	/**
	 * Actually shows the new widget (add it as a child) and cleans the fields.
	 */
	@Override
	protected void resetContext() {
		super.resetContext();
		// Add and lay out the widget correctly.
		Widget newWidget = this.newWidget;
		if (newWidget != null) {
			this.newWidget = null;
			add(newWidget);
			revalidateSubTree();
		}
		this.step = 0;
		this.newScreenshot = null;
		this.isAnimating = false;
	}

	@Override
	protected void updateStep(int step) {
		this.step = step;
		repaint();
	}

}
