/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container.transition;

import ej.microui.display.Display;
import ej.mwt.MWT;
import ej.mwt.Panel;
import ej.mwt.Widget;
import ej.style.container.Rectangle;
import ej.style.util.StyleHelper;

/**
 * A slide transition container slides the widgets during the transition.
 *
 * @since 2.3.0
 */
public class SlideTransitionContainer extends TransitionContainer {

	private final int direction;
	private final boolean overlap;

	private boolean forward;
	private int distance;

	/**
	 * Creates a slide transition manager.
	 * <p>
	 * The given direction is the way the widgets slide when forward. For example, passing {@link MWT#UP}, the new
	 * widget starts below and goes upward.
	 *
	 * @param direction
	 *            one of {@link MWT#LEFT}, {@link MWT#RIGHT}, {@link MWT#TOP}, {@link MWT#BOTTOM}.
	 * @param overlap
	 *            <code>true</code> if the new widget overlaps the previous one, <code>false</code> if the new widget
	 *            pushes the previous one.
	 */
	public SlideTransitionContainer(int direction, boolean overlap) {
		// Validate given direction.
		switch (direction) {
		case MWT.UP:
		case MWT.DOWN:
		case MWT.LEFT:
		case MWT.RIGHT:
			break;
		default:
			throw new IllegalArgumentException();
		}
		this.direction = direction;
		this.overlap = overlap;
	}

	@Override
	public void show(Widget widget, boolean forward) {
		// /!\ Beware that if 'direction' or 'overlap' can be changed between animation, the context of the previous
		// animation must be saved in order to finalize it correctly!
		cancelAnimation();

		Panel panel = getPanel();
		boolean shown = panel != null && panel.isShown();
		int widgetsCount = getWidgetsCount();
		if (!shown || widgetsCount == 0) {
			// It may not be shown, but have widgets, remove them first.
			removeAllWidgets();

			add(widget);
			setChildBounds();
			repaint();
		} else {
			if (widget.getPanel() != null || widget.getParent() != null) {
				throw new IllegalArgumentException();
			}
			Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
			Rectangle contentBounds = StyleHelper.computeContentBounds(bounds, getStyle());
			int contentX = contentBounds.getX();
			int contentY = contentBounds.getY();
			int contentWidth = contentBounds.getWidth();
			int contentHeight = contentBounds.getHeight();

			boolean newIsBelow = newIsBelow(forward);
			int distance;
			switch (getActualDirection(forward)) {
			case MWT.UP:
				distance = -contentHeight;
				if (!newIsBelow) {
					contentY -= distance;
				}
				break;
			case MWT.DOWN:
				distance = contentHeight;
				if (!newIsBelow) {
					contentY -= distance;
				}
				break;
			case MWT.LEFT:
				distance = -contentWidth;
				if (!newIsBelow) {
					contentX -= distance;
				}
				break;
			case MWT.RIGHT:
				distance = contentWidth;
				if (!newIsBelow) {
					contentX -= distance;
				}
				break;
			default:
				// Cannot occur.
				throw new IllegalArgumentException();
			}

			start(widget, forward, contentX, contentY, contentWidth, contentHeight, newIsBelow, distance);

			this.distance = distance;
		}
	}

	private void start(final Widget widget, final boolean forward, final int contentX, final int contentY,
			final int contentWidth, final int contentHeight, final boolean newIsBelow, final int distance) {
		Display.getDefaultDisplay().callSerially(new Runnable() {
			@Override
			public void run() {
				SlideTransitionContainer.this.forward = forward;
				final Widget previousWidget = getWidget(0);
				add(widget);
				widget.validate(contentWidth, contentHeight);
				widget.setBounds(contentX, contentY, contentWidth, contentHeight);
				if (newIsBelow) {
					remove(previousWidget);
					add(previousWidget);
				}
				startAnimation(widget, previousWidget, forward, 0, distance);
			}
		});
	}

	@Override
	protected void resetContext() {
		super.resetContext();
		// Ensure there is only one widget left.
		Display.getDefaultDisplay().callSerially(new Runnable() {
			@Override
			public void run() {
				if (getWidgetsCount() == 2) {
					int indexToRemove;
					if (newIsBelow()) {
						indexToRemove = 1;
					} else {
						indexToRemove = 0;
					}
					final Widget widget = getWidget(indexToRemove);
					remove(widget);
					// Lay out the remaining widget correctly.
					setChildBounds();
				}
			}
		});
		repaint();
	}

	@Override
	protected void updateStep(final int step) {
		Display.getDefaultDisplay().callSerially(new Runnable() {
			@Override
			public void run() {
				moveWidget(step);
			}
		});
		repaint();
	}

	private void moveWidget(int step) {
		if (getWidgetsCount() != 2) {
			// Animation cancelled.
			return;
		}
		Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
		Rectangle contentBounds = StyleHelper.computeContentBounds(bounds, getStyle());

		int stopValue = this.distance;
		// Move widgets.
		Widget firstWidget = getWidget(0);
		Widget secondWidget = getWidget(1);
		switch (this.direction) {
		case MWT.LEFT:
		case MWT.RIGHT:
			int contentX = contentBounds.getX();
			int widgetY = firstWidget.getY();
			// The first one remains still when overlap.
			if (!this.overlap) {
				firstWidget.setLocation(contentX + step, widgetY);
			}
			// When the new one is below, the widgets order is reversed.
			if (newIsBelow()) {
				secondWidget.setLocation(contentX + step, widgetY);
			} else {
				secondWidget.setLocation(contentX + step - stopValue, widgetY);
			}
			break;
		case MWT.UP:
		case MWT.DOWN:
			int widgetX = firstWidget.getX();
			int contentY = contentBounds.getY();
			// The first one remains still when overlap.
			if (!this.overlap) {
				firstWidget.setLocation(widgetX, contentY + step);
			}
			// When the new one is below, the widgets order is reversed.
			if (newIsBelow()) {
				secondWidget.setLocation(widgetX, contentY + step);
			} else {
				secondWidget.setLocation(widgetX, contentY + step - stopValue);
			}
			break;
		}
	}

	// The new one is below if overlap and not forward.
	private boolean newIsBelow() {
		return newIsBelow(this.forward);
	}

	private boolean newIsBelow(boolean forward) {
		return this.overlap && !forward;
	}

	private int getActualDirection(boolean forward) {
		if (forward) {
			return this.direction;
		} else {
			switch (this.direction) {
			case MWT.UP:
				return MWT.DOWN;
			case MWT.DOWN:
				return MWT.UP;
			case MWT.LEFT:
				return MWT.RIGHT;
			case MWT.RIGHT:
				return MWT.LEFT;
			default:
				// Cannot occur.
				throw new IllegalArgumentException();
			}
		}
	}

}
