/*
 * Java
 *
 * Copyright 2015-2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 */
package ej.util.filemanager;

import java.io.IOException;
import java.io.InputStream;

public interface FilesManager {

	/**
	 * Initialize this files manager.
	 *
	 * @throws IOException
	 *             If an I/O errors is occur.
	 */
	public void initialize() throws IOException;

	/**
	 * Shutdown this files manager.
	 *
	 * @throws IOException
	 *             If an I/O errors is occur.
	 */
	public void shutdown() throws IOException;

	/**
	 * Remove all contents from this files manager.
	 *
	 * @throws IOException
	 *             If an I/O errors is occur.
	 */
	public void clear() throws IOException;

	/**
	 * Writes the given contents to specified file.
	 * <p>
	 * The specified path is relative the root directory. The method creates all necessary parent directories.
	 *
	 * @param dirPath
	 *            parent directory of the file to be created.
	 * @param name
	 *            name of the file to be created.
	 * @param contents
	 *            contents of the file to be created.
	 * @throws IOException
	 *             If an I/O errors is occur.
	 */
	public void write(String dirPath, String name, InputStream contents) throws IOException;

	/**
	 * Tests whether a data exists for this filename.
	 *
	 * @param dirPath
	 *            parent directory of the file to be created.
	 * @param filename
	 *            the filename to check.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @return <code>true</code> if the given ID exists, <code>false</code> otherwise
	 */
	boolean exists(String dirPath, String filename) throws IOException;

	/**
	 * Moves a file as a new file.
	 * <p>
	 * If a file is already stored with the given destination filename, it is overwritten. The given source filename
	 * must denote a file.
	 *
	 * @param dirPath
	 *            parent directory of the file to be created.
	 * @param src
	 *            the filename of the file to move.
	 * @param dst
	 *            the filename destination of the new file.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws IllegalArgumentException
	 *             if the given source filename is not a valid identifier.
	 * @throws NullPointerException
	 *             if one of the given parameters is {@code null}.
	 */
	public void move(String dirPath, String src, String dst) throws IOException;

	/**
	 * Reads the content of the specified file.
	 *
	 * @param dirPath
	 *            parent directory of the file to be read.
	 * @param name
	 *            name of the file to be read.
	 * @return an input stream with the content of the specified file, or {@code null} if there no content.
	 */
	public InputStream read(String dirPath, String name);

	/**
	 * This method deletes the specified file.
	 *
	 * @param dirPath
	 *            parent directory of the file to be deleted.
	 * @param name
	 *            name of the file to be deleted.
	 *
	 * @return <code>true</code> if the file was deleted, <code>false</code> otherwise.
	 * @throws IOException
	 *             if the file does not exist
	 */
	public boolean delete(String dirPath, String name) throws IOException;

	/**
	 * Returns an array of strings naming the files in the given directory.
	 *
	 * @param dirPath
	 *            the directory to list its contents.
	 *
	 * @return an array of strings .
	 * @throws IOException
	 *             if the file does not exist
	 */
	public String[] list(String dirPath) throws IOException;
}
